@@ -65,12 +65,21 @@ namespace Banter
 		private Person person;
 		private PersonCardSize cardSize;
 		private Gtk.Image image;
+		private Gtk.Label nameLabel;
 		private ProgressBar progressBar;
 		private Label progressLabel;
+		private HBox widgetColumns;
+		private HBox removeBox;
 		private HBox actionBox;
 		private Gtk.Button textButton;
+		private Gtk.Button audioButton;
 		private Gtk.Button videoButton;
+		private Gtk.Button authorizeButton;
+		private Gtk.Button removeButton;
 		private Gtk.Label statusLabel;
+		private bool showRemoveButton;
+		private bool removeRequested;
+		private Tooltips toolTips;
 		#endregion
 
 
@@ -82,11 +91,18 @@ namespace Banter
 		{
 			get { return person; }
 			set {
-				Logger.Debug ("FIXME: Implement PersonCard.Person [set] - Do we need to do anything here?");
-				// person = value;
+				SetPerson (value);
 			}
 		}
 		
+		public bool ShowRemoveButton
+		{
+			get { return showRemoveButton; }
+			set {
+				showRemoveButton = value;
+				UpdateRemoveButton();
+			}
+		}
 		
 		///<summary>
 		///	The size of the card to be rendered
@@ -112,6 +128,14 @@ namespace Banter
 			person.AvatarUpdated += OnPersonAvatarUpdated;
 			Init();
 		}
+
+		public PersonCard()
+		{
+			this.person = null;
+			this.cardSize = PersonCardSize.Small;
+
+			Init();
+		}
 		#endregion		
 		
 		
@@ -121,6 +145,8 @@ namespace Banter
 		///</summary>		
 		private void Init()
 		{
+			toolTips = new Tooltips();
+			removeRequested = false;
 			this.BorderWidth = 0;
 			//this.Relief = Gtk.ReliefStyle.None;
 			this.CanFocus = false;
@@ -130,9 +156,12 @@ namespace Banter
 			this.BorderWidth = 0;
 			
 			VBox widgetVBox = new VBox (false, 4);
-	        HBox widgetColumns = new HBox(false, 10);
+	        widgetColumns = new HBox(false, 10);
+
+			removeBox = new HBox(false, 5);
+			widgetColumns.PackStart(removeBox, false, false, 0);
 			
-			if(person.Photo != null)
+			if( (person != null) && (person.Photo != null))
 				image = new Gtk.Image(person.Photo.ScaleSimple(32, 32, InterpType.Bilinear));
 			else
 				image = new Gtk.Image(Utilities.GetIcon("blank-photo-128", 32));			
@@ -142,15 +171,14 @@ namespace Banter
 			// Set up the name and status labels
 			VBox nameVBox = new VBox();
 			widgetColumns.PackStart(nameVBox, true, true, 0);
-			Label label = new Label();
-			label.Justify = Gtk.Justification.Left;
-            label.SetAlignment (0.0f, 0.5f);
-			label.LineWrap = false;
-			label.UseMarkup = true;
-			label.UseUnderline = false;
-			label.Markup = string.Format ("<span weight=\"bold\" size=\"medium\">{0}</span>",
-											person.DisplayName);
-			nameVBox.PackStart(label, true, true, 0);
+			nameLabel = new Label();
+			nameLabel.Justify = Gtk.Justification.Left;
+            nameLabel.SetAlignment (0.0f, 0.5f);
+			nameLabel.LineWrap = false;
+			nameLabel.UseMarkup = true;
+			nameLabel.UseUnderline = false;
+			UpdateName();
+			nameVBox.PackStart(nameLabel, true, true, 0);
 
 			statusLabel = new Label();
 			statusLabel.Justify = Gtk.Justification.Left;
@@ -159,13 +187,9 @@ namespace Banter
 			statusLabel.UseUnderline = false;
 
 			statusLabel.LineWrap = false;
-			
-			if (person.PresenceMessage.Length > 0) {
-				statusLabel.Markup = String.Format("<span style=\"italic\" size=\"small\">{0}</span>", person.PresenceMessage);
-			}
-			else {
-				statusLabel.Markup = String.Format("<span style=\"italic\" size=\"small\">{0}</span>", Presence.GetStatusString(person.Presence.Type));
-			}
+
+			UpdateStatus();
+
 			nameVBox.PackStart(statusLabel, true, true, 0);
 
 			actionBox = new HBox(false, 0);
@@ -192,8 +216,8 @@ namespace Banter
 			progressLabel.Ellipsize = Pango.EllipsizeMode.End;
 			widgetVBox.PackStart (progressLabel, false, false, 0);
 
-
-			OnPersonPresenceUpdated (this.person);
+			if(person != null)
+				OnPersonPresenceUpdated (person);
 	        
 	        widgetVBox.ShowAll ();
 	        Add(widgetVBox);
@@ -209,6 +233,120 @@ namespace Banter
 						 Gdk.DragAction.Copy );
 		}
 
+		private void SetPerson (Person person)
+		{
+			if(this.person != null) {
+				this.person.PresenceUpdated -= OnPersonPresenceUpdated;
+				this.person.AvatarUpdated -= OnPersonAvatarUpdated;
+			}
+			this.person = person;
+			this.person.PresenceUpdated += OnPersonPresenceUpdated;
+			this.person.AvatarUpdated += OnPersonAvatarUpdated;
+
+			if(person.Photo != null)
+				image.Pixbuf = person.Photo.ScaleSimple(32, 32, InterpType.Bilinear);
+
+			UpdateName();
+
+			OnPersonPresenceUpdated (person);
+		}
+
+
+		///<summary>
+		///	Updates the showing of the removal button.
+		///</summary>
+		private void UpdateRemoveButton()
+		{
+			if(showRemoveButton) {
+				if(removeButton == null) {
+					Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("gtk-stop", 24));
+					removeButton = new Gtk.Button();
+					removeButton.BorderWidth = 0;
+					removeButton.Relief = Gtk.ReliefStyle.None;
+					removeButton.CanFocus = false;
+					removeButton.Clicked += OnRemoveClicked;
+					removeButton.Image = actionImage;
+					removeBox.PackStart(removeButton, false, false, 0);
+					removeButton.Show();
+					toolTips.SetTip(removeButton,
+							Catalog.GetString("Remove this person"),
+							Catalog.GetString("Remove this person"));
+				}
+				// if a remove has been requested already for this user, make the button not sensitive
+				removeButton.Sensitive = !removeRequested;
+			} else {
+				if(removeButton != null){
+					removeBox.Remove(removeButton);
+					removeButton = null;
+				}				
+			}
+		}
+
+		///<summary>
+		///	Updates the formatting of the name for presence etc.
+		///</summary>
+		private void UpdateName()
+		{
+			if(person != null) {
+				if (person.ProviderUser.Relationship == ProviderUserRelationship.ReceivedInvitation)
+					nameLabel.Markup = string.Format ("<span weight=\"bold\" size=\"medium\">{0}</span>",
+											person.DisplayName);
+				else if (person.ProviderUser.Relationship == ProviderUserRelationship.SentInvitation)
+					nameLabel.Markup = string.Format ("<span weight=\"bold\" size=\"medium\">{0}</span>",
+											person.DisplayName);
+				else if(person.Presence.Type == PresenceType.Offline)
+					nameLabel.Markup = string.Format ("<span foreground=\"grey\" weight=\"bold\" size=\"medium\">{0}</span>",
+											person.DisplayName);
+				else
+					nameLabel.Markup = string.Format ("<span weight=\"bold\" size=\"medium\">{0}</span>",
+											person.DisplayName);
+			} else {
+				nameLabel.Markup = "<span weight=\"bold\" size=\"medium\">Unknown</span>";
+			}
+		}
+
+		///<summary>
+		///	Updates the formatting of the name for presence etc.
+		///</summary>
+		private void UpdateStatus()
+		{
+			string presenceMessage;
+			string presenceColor = "black";
+
+			if(person != null) {
+				if (person.ProviderUser.Relationship == ProviderUserRelationship.SentInvitation) {
+					presenceMessage = Catalog.GetString("Invited");
+					presenceColor = "#373935";
+				} else if (person.ProviderUser.Relationship == ProviderUserRelationship.ReceivedInvitation) {
+					presenceMessage = Catalog.GetString("Requesting");
+					presenceColor = "#373935";
+				} else {
+					if (person.PresenceMessage.Length > 0)
+						presenceMessage = person.PresenceMessage;
+					else
+						presenceMessage = Presence.GetStatusString(person.Presence.Type);
+
+					switch(person.Presence.Type) {
+						case PresenceType.Offline:
+							presenceColor = "grey";
+							break;
+						case PresenceType.Available:
+							presenceColor = "darkgreen";
+							break;
+						case PresenceType.Away:
+						case PresenceType.Busy:
+							presenceColor = "brown";
+							break;
+					}
+				}
+			} else {
+				presenceMessage = Presence.GetStatusString(PresenceType.Offline);
+				presenceColor = "grey";
+			}
+
+			statusLabel.Markup = String.Format("<span foreground=\"{0}\" style=\"italic\" size=\"small\">{1}</span>", presenceColor, presenceMessage);
+		}
+
 
 		///<summary>
 		///	Handles Avatar Events on a Person
@@ -216,7 +354,8 @@ namespace Banter
 		private void OnPersonAvatarUpdated (Person person)
 		{
 			// Logger.Debug("Updating presence on {0}", person.DisplayName);
-			//RenderWidget();
+			if(person.Photo != null)
+				image.Pixbuf = person.Photo.ScaleSimple(32, 32, InterpType.Bilinear);
 		}	
 
 		///<summary>
@@ -225,54 +364,102 @@ namespace Banter
 		private void OnPersonPresenceUpdated (Person person)
 		{
 			//Logger.Debug("OnPersonPresenceUpdated on {0}", person.DisplayName);
+			UpdateName();
+			UpdateStatus();
 
-			if (person.PresenceMessage.Length > 0) {
-				statusLabel.Markup = String.Format("<span style=\"italic\" size=\"small\">{0}</span>", person.PresenceMessage);
-			}
-			else {
-				statusLabel.Markup = String.Format("<span style=\"italic\" size=\"small\">{0}</span>", Presence.GetStatusString(person.Presence.Type));
-			}
-
-			// Add capabilities icons if they have any capabilities
-			// change this later to show their capabilities when we actually have them
-			if(person.Presence.Type != PresenceType.Offline) {
-				if(videoButton == null) {
-					Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("webcam", 24));
-
-					videoButton = new Gtk.Button();
-					videoButton.BorderWidth = 0;
-					videoButton.Relief = Gtk.ReliefStyle.None;
-					videoButton.CanFocus = false;
-					videoButton.Image = actionImage;
-					videoButton.Clicked += OnVideoChatClicked;
-					actionBox.PackEnd(videoButton, false, false, 0);
-					videoButton.Show();
-				}
-				
-				if(textButton == null) {
-					Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("text", 24));
-					textButton = new Gtk.Button();
-					textButton.BorderWidth = 0;
-					textButton.Relief = Gtk.ReliefStyle.None;
-					textButton.CanFocus = false;
-					textButton.Clicked += OnTextChatClicked;
-					textButton.Image = actionImage;
-					actionBox.PackEnd(textButton, false, false, 0);
-					textButton.Show();
+			if (person.ProviderUser.Relationship == ProviderUserRelationship.ReceivedInvitation) {
+				if(authorizeButton == null) {
+					Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("add", 24));
+					authorizeButton = new Gtk.Button();
+					authorizeButton.BorderWidth = 0;
+					authorizeButton.Relief = Gtk.ReliefStyle.None;
+					authorizeButton.CanFocus = false;
+					authorizeButton.Clicked += OnAuthorizeClicked;
+					authorizeButton.Image = actionImage;
+					actionBox.PackEnd(authorizeButton, false, false, 0);
+					authorizeButton.Show();
+					toolTips.SetTip(authorizeButton,
+							Catalog.GetString("Authorize this person"),
+							Catalog.GetString("Authorize this person to chat with you"));
 				}
+			} else if (person.ProviderUser.Relationship == ProviderUserRelationship.SentInvitation) {
+				// do nothing, there is already a remove button available
 			} else {
-				if(textButton != null) {
-					actionBox.Remove(textButton);
-					textButton = null;
-				}
-				if(videoButton != null) {
-					actionBox.Remove(videoButton);
-					videoButton = null;
+				// Add capabilities icons if they have any capabilities
+				// change this later to show their capabilities when we actually have them
+				if(person.Presence.Type != PresenceType.Offline) {
+					if(videoButton == null) {
+						Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("webcam", 24));
+
+						videoButton = new Gtk.Button();
+						videoButton.BorderWidth = 0;
+						videoButton.Relief = Gtk.ReliefStyle.None;
+						videoButton.CanFocus = false;
+						videoButton.Image = actionImage;
+						videoButton.Clicked += OnVideoChatClicked;
+						actionBox.PackEnd(videoButton, false, false, 0);
+						videoButton.Show();
+						toolTips.SetTip(videoButton,
+							Catalog.GetString("Start a video chat"),
+							Catalog.GetString("Start a video chat"));
+					}
+
+					if(audioButton == null) {
+						Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("mic", 24));
+						audioButton = new Gtk.Button();
+						audioButton.BorderWidth = 0;
+						audioButton.Relief = Gtk.ReliefStyle.None;
+						audioButton.CanFocus = false;
+						audioButton.Image = actionImage;
+						audioButton.Clicked += OnAudioChatClicked;
+						actionBox.PackEnd(audioButton, false, false, 0);
+						audioButton.Show();
+						toolTips.SetTip(audioButton,
+							Catalog.GetString("Start an audio chat"),
+							Catalog.GetString("Start an audio chat"));
+					}
+					
+					if(textButton == null) {
+						Gtk.Image actionImage = new Gtk.Image(Utilities.GetIcon("text", 24));
+						textButton = new Gtk.Button();
+						textButton.BorderWidth = 0;
+						textButton.Relief = Gtk.ReliefStyle.None;
+						textButton.CanFocus = false;
+						textButton.Clicked += OnTextChatClicked;
+						textButton.Image = actionImage;
+						actionBox.PackEnd(textButton, false, false, 0);
+						textButton.Show();
+						toolTips.SetTip(textButton,
+							Catalog.GetString("Start a text chat"),
+							Catalog.GetString("Start a text chat"));
+					}
+
+					// Make sure to remove these, the user may have just been added
+					if(authorizeButton != null){
+						actionBox.Remove(authorizeButton);
+						authorizeButton = null;
+					}
+				} else {
+					if(textButton != null) {
+						actionBox.Remove(textButton);
+						textButton = null;
+					}
+					if(audioButton != null) {
+						actionBox.Remove(audioButton);
+						audioButton = null;
+					}
+					if(videoButton != null) {
+						actionBox.Remove(videoButton);
+						videoButton = null;
+					}
+					if(authorizeButton != null){
+						actionBox.Remove(authorizeButton);
+						authorizeButton = null;
+					}
 				}
 			}
 		}
 
-	
 		
 		
 		private void DragDataReceivedHandler (object o, DragDataReceivedArgs args)
@@ -379,12 +566,66 @@ namespace Banter
 			});
 		}
 */
+		///<summary>
+		///	When user clicks to authorize a person
+		///</summary>
+		private void OnAuthorizeClicked (object o, EventArgs args)
+		{
+			authorizeButton.Sensitive = false;
+			statusLabel.Markup = String.Format("<span foreground=\"#373935\" style=\"italic\" size=\"small\">{0}</span>",
+					Catalog.GetString("Authorizing..."));
+			person.ProviderUser.Authorize (String.Empty);
+		}
+
+		///<summary>
+		///	When user clicks to remove a currently authorized person
+		///</summary>
+		private void OnRemoveClicked (object o, EventArgs args)
+		{
+			removeRequested = true;
+			Logger.Debug("OnRemoveClicked called for {0} with relationship {1}", person.DisplayName, person.ProviderUser.Relationship);
+			if (person.ProviderUser.Relationship == ProviderUserRelationship.ReceivedInvitation) {
+				Logger.Debug("Relationship is RecievedInvitation, calling DenyAuthorization()");
+				person.ProviderUser.DenyAuthorization (String.Empty);
+			} else if (person.ProviderUser.Relationship == ProviderUserRelationship.SentInvitation) {
+				Logger.Debug("Relationship is SentInvitation, calling RevokeInvitation()");
+				person.ProviderUser.RevokeInvitation();
+			} else {
+				Logger.Debug("Relationship is only other case, calling RemoveUser()");
+				person.ProviderUser.RemoveUser();
+			}
+			removeButton.Sensitive = !removeRequested;
+			statusLabel.Markup = String.Format("<span foreground=\"#373935\" style=\"italic\" size=\"small\">{0}</span>",
+					Catalog.GetString("Removing..."));
+			if(textButton != null) {
+				actionBox.Remove(textButton);
+				textButton = null;
+			}
+			if(audioButton != null) {
+				actionBox.Remove(audioButton);
+				audioButton = null;
+			}
+			if(videoButton != null) {
+				actionBox.Remove(videoButton);
+				videoButton = null;
+			}
+			if(authorizeButton != null){
+				actionBox.Remove(authorizeButton);
+				authorizeButton = null;
+			}
+		}
+
 
 		private void OnTextChatClicked (object o, EventArgs args)
 		{
 			ChatWindowManager.InitiateChat(person, ChatType.Text);
 		}
 
+		private void OnAudioChatClicked (object o, EventArgs args)
+		{
+			ChatWindowManager.InitiateChat(person, ChatType.Audio);
+		}
+
 		private void OnVideoChatClicked (object o, EventArgs args)
 		{
 			ChatWindowManager.InitiateChat(person, ChatType.Video);
